#! /usr/bin/perl

# ============================================================== #
#  Fumy RSS Feeds Merger   Ver 2.0.0             [mergerss.cgi]  #
# -------------------------------------------------------------- #
#  Copyright (C) Fumihiro Nishimura.(Nishishi) 2007-2016.        #
#                                                                #
#  指定した複数のRSSフィード(またはAtomフィード)を結合させて     #
#  1つの大きなRSSフィード(またはAtomフィード)を作成するツール。  #
#  結果は直接表示することもファイルに出力することもできます。    #
# -------------------------------------------------------------- #
#  必要なモジュール                                              #
#   ●XML::TreePP                                                #
#   ●XML::FeedPP                                                #
#                                                                #
#  これらのモジュールがない場合は、モジュールをCPANなどから入手  #
#  して、このCGIと同じディレクトリに以下のように格納して下さい。 #
#                                                                #
#  《ベースディレクトリ》                                        #
#  　　├─ mergerss.cgi (このCGI)                               #
#  　　├─ データファイル群（sample1.datなど）                  #
#    　└─ XML《サブディレクトリ》                              #
#    　　　　├─ TreePP.pm                                      #
#    　　　　└─ FeedPP.pm                                      #
#                                                                #
#  ※XML::TreePPモジュールとXML::FeedPPモジュールはたいてい存在  #
#  しないので、別途入手する必要があるでしょう。ppmを使ってサーバ #
#  にインストールできるならそうして下さい。無理なら、上記のよう  #
#  に直接 TreePP.pm と FeedPP.pm ファイルを配置して下さい。      #
# ============================================================== #

# ================ #
#  ▼ユーザ設定▼  #（どれも任意。必須ではありません。）
# ================ #
my $default_feed_type = 'RSS';			# デフォルトのフィード種類＝[ RSS , RDF , Atom ]
my $default_output    = 'show';			# デフォルトの出力方法　　＝[ show , file , file-ping ]（showだと表示／fileだとファイル化／file-pingだとファイル化した後でPingサーバの応答を模倣）
my $default_charcode  = 'UTF-8';		# デフォルトの文字コード　＝[ Shift_JIS , UTF-8 ]

my $cgi_charcode = 'UTF-8';			# このCGI自体の文字コード（※公式サイト配布時点ではUTF-8）

# ---------------------------------- #
# ▼これ以降に設定項目はありません▼ #
# ---------------------------------- #
unshift( @INC, "." );
#use strict;
#use warnings;
use XML::TreePP;
use XML::FeedPP;

my $param = &getparam();

if( $param eq '' ) {
	# ガイド表示
	&guide();
}
elsif( $param =~ m/\.\./i || $param =~ m/^\//i ) {
	# 上位階層へのアクセスは禁止
	&errorexit("指定ファイルが見つからないか、または禁止されているファイルパスが指定されました。（CGIの設置位置よりも上位階層のファイルは指定できません。）");
}
else {
	# マージの実行
	&margework($param);
}
exit;

# ---------- #
# メイン処理 #
# ---------- #
sub margework
{
	my $listfile = shift @_;
	my @margelist;

	my $output   = $default_output;
	my $charcode = $default_charcode;
	my $feedtype = $default_feed_type;

	# リストファイルを開く
	if( open(IN,"$listfile") ) {
		# リストを得る
		foreach my $oneline ( <IN> ) {
			$oneline =~ s/[\r\n]+\z//;	# 改行コードを消す。
			if(    $oneline =~ m/^@(.*)$/ ) { $output = $1; }		# 出力先
			elsif( $oneline =~ m/^%(.*)$/ ) { $charcode = $1; }		# 文字コード
			elsif( $oneline =~ m/^#(.*)$/ ) { $feedtype = $1; }		# フィード種類
			else {
				push( @margelist, $oneline );
			}
		}
		close(IN);
	}
	else {
		# 読めなかったらエラー
		&errorexit( &forsafety("データファイル $listfile が読めませんでした。存在や権限を確認して下さい。") );
	}

	# フィードの種類を選択(RSS/RDF/Atom)
	my $feed;
	if( $feedtype =~ /RSS/i ) {
		$feed = XML::FeedPP::RSS->new();
	}
	elsif( $feedtype =~ /RDF/i ) {
		$feed = XML::FeedPP::RDF->new();
	}
	else {
		$feed = XML::FeedPP::Atom->new();
	}

	# フィードをマージ
	foreach my $onefeed ( @margelist ) {
		$feed->merge( $onefeed );
	}
	$feed->pubDate(time());

	# 結果出力or表示
	if( $output =~ /^file/i ) {
		# ファイル出力
		my $feedfile = &makefilename($listfile);
		$feed->to_file( $feedfile, $charcode );
		# 結果表示の分岐
		if( $output =~ /ping$/i ) {
			# Ping応答
			print "Content-type: text/xml\n\n";
			print &outputresult($feedfile);
		}
		else {
			# 普通に結果を表示
			$feedfile = &forsafety( $feedfile );
			my $resbody = qq|<p>ファイルへの出力を完了しました。<br>出力先：<a href="$feedfile" target="_blank">$feedfile</a></p>|;
			my $guidemsg = q|<p align="center"><a href="?">CGI TOPへ戻る</a></p><hr><p class="note">※出力タイプを「file」から「file-ping」に変更すると、ファイル生成時にPingサーバの応答を模倣して返すので、ブログツールなどのUpdate Ping送信先に設定することができます。（ブログの更新とRSS統合を同期させたい場合に使えます。）<br>※出力タイプを「file」から「show」に変更すると、生成したRSS（Atom）をそのまま返します。常に動的に統合したい場合にどうぞ。</p>|;
			&showdisplay( '出力結果', '', $resbody . $guidemsg, '', $cgi_charcode );
		}
	}
	else {
		# 表示
		print "Content-type: text/xml\n\n";
		print $feed->to_string( $charcode );
	}
	
}

# ---------------------------------------------- #
# ファイル生成時の出力結果表示（Ping応答を模倣） #
# ---------------------------------------------- #
sub outputresult
{
	my $filename = &forsafety( shift @_ );

	my $resxml = << "EOX";
<?xml version="1.0" encoding="$cgi_charcode"?>
<methodResponse>
	<params>
		<param>
			<value><struct>
				<member>
					<name>flerror</name>
					<value>
					<boolean>0</boolean>
					</value>
				</member>
				<member>
					<name>message</name>
					<value><string>
						Output was completed! - File:[ $filename ]
					</string></value>
				</member>
			</struct></value>
		</param>
	</params>
</methodResponse>
EOX

	return $resxml;
}

# ---------------------- #
# 出力用ファイル名の作成 #	引数[1]:データファイル名
# ---------------------- #	データファイル名の拡張子を.xmlに変更して返す。なんか無理だったらdefault.xmlにする。
sub makefilename
{
	my $datname = shift @_;
	my $feedfile;

	if( $datname =~ m/(.+)\..*/ ) {
		$feedfile = $1 . '.xml';
	}
	else {
		$feedfile = 'default.xml';
	}

	return $feedfile;
}

# ---------------- #
# パラメータの取得 #
# ---------------- #
sub getparam
{
	my $query = $ENV{'QUERY_STRING'};

	if( $query ne '' ) {
		# クエリがあれば
		if( -f $query ) {
			# 指定されたファイルが存在していれば
			return $query;
		}
		else {
			# 存在していなければ
			return '/';
		}
	}
	else {
		# クエリがなければ
		return '';
	}
}

# ---------- #
# ガイド表示 #
# ---------- #
sub guide
{
	my $list = '<ul class="list">' . join("",&getfilelist('dat')) . '</ul>';
	my $guidebody1 = 'Fumy RSS Feeds Mergerは、複数のRSS/Atomフィードを結合させて1つの大きなRSSフィードを生成するCGIです。<br>現在、このディレクトリ内にある結合リスト（データファイル）は以下の通りです。<br>';
	my $guidebody2 = 'ここにデータファイル名を表示させたくない場合は、サブディレクトリに置くか、データファイルの拡張子を「.dat」以外にして下さい。<br>';

	&showdisplay( 'ガイド', '', $guidebody1 . $list . $guidebody2, '', $cgi_charcode );
}

# -------- #
# 画面表示 #
# -------- #
sub showdisplay
{
	my ( $title, $status, $body, $footer, $charcode ) = @_;
	
	print "Content-type: text/html\n\n";

	my @showhtml = <DATA>;
	foreach my $line (@showhtml) {
		$line =~ s/_WORKTITLE_/$title/g;
		$line =~ s/_STATUS_/$status/g;
		$line =~ s/_BODY_/$body/g;
		$line =~ s/_FOOTER_/$footer/g;
		$line =~ s/_CHARCODE_/$charcode/g;
	}
	print @showhtml;
}

# -------------------------- #
# ディレクトリファイルリスト #	引数[1]:指定拡張子
# -------------------------- #
sub getfilelist
{
	my $ext = shift @_;
	my @html;

	# ディレクトリのファイル一覧を得る
	opendir( DIRECTORY, './' );
	my @filelist = readdir( DIRECTORY );
	closedir( DIRECTORY );
	splice( @filelist, 0, 2 );		# 親DIRと自己DIRを除く

	# ファイルリストを処理
	foreach my $onefile ( @filelist ) {
		# 指定拡張子のファイルのみをリスト
		if( $onefile =~ m/\.$ext$/i ) {
			# リンクを作る
			$onefile = &forsafety($onefile);
			push( @html, qq|<li><a href="?$onefile">$onefileを実行</a> <span class="subdata">［<a href="$onefile">$onefileの中身を見る</a>］</span></li>\n| );
		}
	}

	return @html;
}

# ------ #
# 安全用 #
# -------#
sub forsafety
{
	my $string 		= shift @_;

	# エンコード
	$string =~ s|&|&amp;|g;		# アンドを実体参照に
	$string =~ s|<|&lt;|g;		# 小なりを実体参照に
	$string =~ s|>|&gt;|g;		# 大なりを実体参照に
	$string =~ s|"|&quot;|g;	# 二重引用符を実体参照に
	$string =~ s|'|&apos;|g;	# 引用符を実体参照に

	return $string;
}

# ---------------- #
# エラーメッセージ #
# ---------------- #
sub errorexit
{
	my $msg = shift @_;

	print "Content-type: text/html\n\n";
	print qq|<html lang="ja"><head><title>Fumy RSS Feeds Merger - ERROR</title></head><body>\n|;
	print qq|<div style="background-color:red; color:#fffff0; font-weight:bold; font-family:Arial,sans-serif; padding:1px;"> Fumy RSS Feeds Merger - ERROR </div><br>\n|;
	print qq|<div style="font-size: smaller;">エラー内容：</div>\n|;
	print qq|<div style="border:1px dotted blue; padding:1em;">$msg</div>\n|;
	print qq|<p style="text-align: right; font-size:80%; margin-top:0.5em;"><a href="?">ガイドへ</a></p>|;
	print qq|<div style="font-size: 75%; font-family: Verdana,Arial,sans-serif; border-top: 1px solid #cccccc; padding-top: 0.3em;"><a href="http://www.nishishi.com/">Fumy RSS Feeds Merger</a></div>|;
	print qq|</body></html>\n|;

	exit;
}

__END__
<!DOCTYPE html>
<html lang="ja">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=_CHARCODE_">
	<meta name="viewport" content="initial-scale=1">
	<!-- この部分は、ＣＧＩのコントロールパネル用のＨＴＭＬです。 -->
	<title>_WORKTITLE_ - Fumy RSS Feeds Merger</title>
	<style type="text/css">
		body { background-color: #aaccaa; font-family: "メイリオ",Meiryo,"ヒラギノ角ゴ Pro","Hiragino Kaku Gothic",sans-serif; }
		div#contents { background-color: #ffffff; border: 1px #008000 solid; }
		div#contents a:hover { color: red; }
		div#header { background-color: #ccffcc; color: #000000; border-bottom: 1px green solid; }
		div#header h1 { margin: 0px 0.3em; font-size: 150%; }
		div#header h1 span.appname { font-size: 80%; }
		div#status { margin: 0px; padding: 0.3em 0.5em; text-align: right; font-size: 85%; }
		div#main { margin: 0.1em 1em 1em 1em; }
		div#footer { text-align: center; margin: 1em; }
		p#copyright { margin: 0.3em 0.5em; padding: 0.3em; font-size: 75%; }
		.list { border: 1px solid #ccc; border-radius: 0.5em; margin: 1em 0px; padding: 1.5em 0em 1em 2em; line-height: 1.6; }
		.list li { margin-bottom: 0.5em; }
		.list .subdata { font-size: small; display: inline-block; }
		p.fram { margin: 0.6em; font-size: 80%; line-height: 1.35; font-family: Verdana,sans-serif; }
		.note { font-size: 80%; }
	</style>
</head>
<body>
	<div id="contents">
		<div id="header">
			<h1>_WORKTITLE_<span class="appname"> - Fumy RSS Feeds Merger (Ver 2.0.0)</span></h1>
		</div>
		<div id="status">
			_STATUS_
		</div>
		<div id="main">
			_BODY_
		</div>
		<div id="footer">
			_FOOTER_
		</div>
	</div>
	<p id="copyright">
		<!-- ※以下の著作権表示を削除したり改変したりしないようにお願いします。 -->
		<a href="http://www.nishishi.com/cgi/mergerss/">Fumy RSS Feeds Merger</a> Ver 2.0.0 : 
		Copyright &copy; 2007-2016 <a href="http://www.nishishi.com/">にしし/西村文宏(Nishishi)</a>.
	</p>
	<hr>
	<p class="fram">※ローカルHDD内にあるHTMLなどを更新時刻順に読み取って、RSS（Atom）フィードを自動生成できるWindows用フリーソフト「<a href="http://www.nishishi.com/soft/rssmaker/">Fumy RSS &amp; Atom Maker</a>」もあります。自力で作成したHTMLに対してもRSSフィードを配信したい場合に便利です！ ぜひ、ご活用下さい。</p>
</body>
</html>
